@@ -32,6 +32,7 @@
 
 #include <libgames-support/games-clock.h>
 #include <libgames-support/games-conf.h>
+#include <libgames-support/games-gtk-compat.h>
 #include <libgames-support/games-help.h>
 #include <libgames-support/games-scores.h>
 #include <libgames-support/games-scores-dialog.h>
@@ -423,7 +424,9 @@ main (int argc, char **argv)
   if (!games_runtime_init ("gnotravex"))
     return 1;
 
+#ifdef ENABLE_SETGID
   setgid_io_init ();
+#endif
 
   context = g_option_context_new (NULL);
 #if GLIB_CHECK_VERSION (2, 12, 0)
@@ -501,7 +504,7 @@ main (int argc, char **argv)
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
   gtk_widget_realize (space);
-  bg_gc = gdk_gc_new (space->window);
+  bg_gc = gdk_gc_new (gtk_widget_get_window (space));
   gdk_gc_set_tile (bg_gc, default_background_pixmap);
   gdk_gc_set_fill (bg_gc, GDK_TILED);
 
@@ -608,8 +611,8 @@ create_window (void)
 gint
 expose_space (GtkWidget * widget, GdkEventExpose * event)
 {
-  gdk_draw_drawable (widget->window,
-                     widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+  gdk_draw_drawable (gtk_widget_get_window (widget),
+                     gtk_widget_get_style (widget)->fg_gc[GTK_STATE_NORMAL],
                      buffer, event->area.x, event->area.y,
                      event->area.x, event->area.y,
                      event->area.width, event->area.height);
@@ -1273,8 +1276,7 @@ game_score (void)
   GamesScoreValue score;
 
   if (!have_been_hinted) {
-    seconds = GAMES_CLOCK (timer)->stopped;
-    games_clock_set_seconds (GAMES_CLOCK (timer), (int) seconds);
+    seconds = games_clock_get_seconds (GAMES_CLOCK (timer));
     score.time_double = (gfloat) (seconds / 60) + (gfloat) (seconds % 60) / 100;
     pos = games_scores_add_score (highscores, score);
   }
@@ -1333,14 +1335,14 @@ redraw_all (void)
   guint x, y;
   GdkRegion *region;
 
-  if (!space->window)
+  if (!gtk_widget_get_window (space))
     return;
 
-  region = gdk_drawable_get_clip_region (GDK_DRAWABLE (space->window));
-  gdk_window_begin_paint_region (space->window, region);
+  region = gdk_drawable_get_clip_region (GDK_DRAWABLE (gtk_widget_get_window (space)));
+  gdk_window_begin_paint_region (gtk_widget_get_window (space), region);
 
-  gdk_window_clear (space->window);
-  gdk_draw_rectangle (space->window, bg_gc, TRUE, 0, 0, -1, -1);
+  gdk_window_clear (gtk_widget_get_window (space));
+  gdk_draw_rectangle (gtk_widget_get_window (space), bg_gc, TRUE, 0, 0, -1, -1);
   gdk_draw_rectangle (buffer, bg_gc, TRUE, 0, 0, -1, -1);
   for (y = 0; y < size; y++)
     for (x = 0; x < size * 2; x++)
@@ -1348,7 +1350,7 @@ redraw_all (void)
 
   gui_draw_arrow(buffer);
 
-  gdk_window_end_paint (space->window);
+  gdk_window_end_paint (gtk_widget_get_window (space));
   gdk_region_destroy (region);
 }
 
@@ -1362,13 +1364,13 @@ redraw_left (void)
 
   region = gdk_region_rectangle (&rect);
 
-  gdk_window_begin_paint_region (space->window, region);
+  gdk_window_begin_paint_region (gtk_widget_get_window (space), region);
 
   for (y = 0; y < size; y++)
     for (x = 0; x < size; x++)
       gui_draw_pixmap (buffer, x, y, FALSE);
 
-  gdk_window_end_paint (space->window);
+  gdk_window_end_paint (gtk_widget_get_window (space));
   gdk_region_destroy (region);
 }
 
@@ -1409,16 +1411,16 @@ create_mover (void)
 {
   GdkWindowAttr attributes;
 
-  /* The depth of mover.window must match the depth of space->window. */
+  /* The depth of mover.window must match the depth of gtk_widget_get_window (space). */
   attributes.wclass = GDK_INPUT_OUTPUT;
   attributes.window_type = GDK_WINDOW_CHILD;
   attributes.event_mask = 0;
   attributes.width = tile_size;
   attributes.height = tile_size;
-  attributes.colormap = gdk_drawable_get_colormap (space->window);
-  attributes.visual = gdk_drawable_get_visual (space->window);
+  attributes.colormap = gdk_drawable_get_colormap (gtk_widget_get_window (space));
+  attributes.visual = gdk_drawable_get_visual (gtk_widget_get_window (space));
 
-  mover.window = gdk_window_new (space->window, &attributes,
+  mover.window = gdk_window_new (gtk_widget_get_window (space), &attributes,
                                  (GDK_WA_VISUAL | GDK_WA_COLORMAP));
   mover.pixmap = NULL;
 }
@@ -1534,7 +1536,7 @@ void
 timer_start (void)
 {
   games_clock_stop (GAMES_CLOCK (timer));
-  games_clock_set_seconds (GAMES_CLOCK (timer), 0);
+  games_clock_reset (GAMES_CLOCK (timer));
   games_clock_start (GAMES_CLOCK (timer));
 }
 
@@ -1593,17 +1595,19 @@ create_menu (GtkUIManager * ui_manager)
 void
 make_buffer (GtkWidget * widget)
 {
+  GtkAllocation allocation;
 
   if (buffer)
     g_object_unref (buffer);
 
-  buffer = gdk_pixmap_new (widget->window, widget->allocation.width,
-                           widget->allocation.height, -1);
-
+  gtk_widget_get_allocation (widget, &allocation);
+  buffer = gdk_pixmap_new (gtk_widget_get_window (widget),
+                           allocation.width,
+                           allocation.height, -1);
 }
 
 void
-new_game (){
+new_game (void){
   gchar *str;
 
   /* Reset pause menu */
@@ -1651,7 +1655,7 @@ save_state_cb (EggSMClient *client,
   gint argc;
   gint xpos, ypos;
 
-  gdk_window_get_origin (window->window, &xpos, &ypos);
+  gdk_window_get_origin (gtk_widget_get_window (window), &xpos, &ypos);
 
   argc = 0;
   argv[argc++] = g_get_prgname ();
@@ -1687,7 +1691,7 @@ size_cb (GtkAction * action, gpointer data)
 
   newsize = gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action));
 
-  gdk_drawable_get_size (space->window, &width, &height);
+  gdk_drawable_get_size (gtk_widget_get_window (space), &width, &height);
 
   if (game_state == paused)
     gtk_action_activate (resume_action);
@@ -1978,7 +1982,7 @@ load_default_background (void)
   gdk_pixbuf_render_pixmap_and_mask_for_colormap (pb,
                                                   gdk_colormap_get_system (), 
                                                   &pm, NULL, 127);
-  gdk_pixbuf_unref (pb);
+  g_object_unref (pb);
   g_free (path);
 
   default_background_pixmap = pm; 
